/*
 * Copyright 2020-present Ladifire. All rights reserved.
 * This file is licensed to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License. You may obtain a copy
 * of the License at http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under
 * the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR REPRESENTATIONS
 * OF ANY KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

import * as React from 'react';

import {MessageDynamicList} from './MessageDynamicList';
import {HeightCache} from './height_cache';
import {Layout} from './layout';
import AutoSizer from './AutoSizer';

export interface RowRendererProps {
  index: number;
  hoverProps?: {
    hasFocus: boolean;
    isHovered: boolean;
  }
}

export interface Props {
  defaultRowHeight?: number;
  isLoading: boolean;
  isLoadingOlder: boolean;
  isLoadingNewer: boolean;
  rowRenderer: (props: RowRendererProps) => React.ReactElement;
  keys?: string[];
}

export interface State {
  isLoading: boolean;
  isLoadingOlder: boolean;
  isLoadingNewer: boolean;
}

export interface QueryParams {
  latest?: string;
  oldest?: string;
  limit: number;
  offset: number;
}

export class FullDynamicList extends React.PureComponent<Props, State> {
  static displayName = 'FullDynamicList';
  static defaultProps = {
    defaultRowHeight: 50,
  };

  protected heightCache: HeightCache;
  protected layout: Layout;
  private keys: string[];
  private rows: any[];
  private useAnchor: boolean;
  private keyToScrollTo: string | undefined;

  constructor(props: Props) {
    super(props);

    this.heightCache = new HeightCache({
      DEFAULT_HEIGHT: props.defaultRowHeight,
    });

    this.layout = new Layout({
      heightCache: this.heightCache,
      keys: [],
      stickToBottom: false,
      shouldExcludeFirstItemFromGravity: false,
      STICKY_EPSILON: 0,
      STICKY_EPSILON_SETHEIGHT: 0,
      ANCHOR_OFFSET: 0,
    });

    this.keys = props.keys;
    this.rows = [];

    this.state = {
      isLoading: props.isLoading,
      isLoadingNewer: props.isLoadingNewer,
      isLoadingOlder: props.isLoadingOlder,
    };
  }

  UNSAFE_componentWillReceiveProps(nextProps: Props) {
    this.setState((prevState: State) => {
      if (nextProps.isLoading !== prevState.isLoading) {
        this.setState({isLoading: nextProps.isLoading});
      }
      if (nextProps.isLoadingNewer !== prevState.isLoadingNewer) {
        this.setState({isLoadingNewer: nextProps.isLoadingNewer});
      }
      if (nextProps.isLoadingOlder !== prevState.isLoadingOlder) {
        this.setState({isLoadingOlder: nextProps.isLoadingOlder});
      }
    });
  }

  renderRow = (index: number, a?: {hasFocus: boolean; isHovered: boolean}) => {
    return this.props.rowRenderer({
      index,
      hoverProps: a,
    });
  };

  render() {
    return (
      <AutoSizer style={{width: '100%', height: '100%'}}>
        {({width, height}) => {
          return (
            <MessageDynamicList
              layout={this.layout}
              width={width}
              height={height}
              keys={this.keys}
              rowRenderer={this.renderRow}
            />
          )
        }}
      </AutoSizer>
    );
  }
}
